#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
PyFingerprint
Copyright (C) 2015 Bastian Raschke <bastian.raschke@posteo.de>
All rights reserved.

@author: Bastian Raschke <bastian.raschke@posteo.de>

@revisioned 20170327 by Sven Gächter (sven@familiegaechter.ch) to interact with DoorPi to unlock Doors

"""

import hashlib
from pyfingerprint.pyfingerprint import PyFingerprint
import time , requests
import os
os.environ['http_proxy']=''
import urllib2


#Timeout for Reader unread
t_end = time.time() + 10
#print(time.time())
#print(t_end)

## Search for a finger


## Tries to initialize the sensor
try:
    f = PyFingerprint('/dev/ttyUSB0', 57600, 0xFFFFFFFF, 0x00000000)

    if ( f.verifyPassword() == False ):
        raise ValueError('The given fingerprint sensor password is wrong!')

except Exception as e:
    print('The fingerprint sensor could not be initialized!')
    print('Exception message: ' + str(e))
    exit(1)

## Gets some sensor information
print('Currently stored templates: ' + str(f.getTemplateCount()))

## Tries to search the finger and calculate hash
try:
    print('Waiting for finger...')

    ## Wait that finger is read
    while time.time() < t_end and ( f.readImage() == False ):
				pass
    ## Converts read image to characteristics and stores it in charbuffer 1
    f.convertImage(0x01)

    ## Searchs template
    result = f.searchTemplate()

    positionNumber = result[0]
    accuracyScore = result[1]

    if ( positionNumber == -1 ):
        print('No match found!')
		# Transaction to DoorPi: No Finger listed or too long- Timeout.  // Please modify URL below with your credentials e.g: http://door:pi@doorpi:8080
        r = requests.get('http://username:password@doorpi-url:port/control/trigger_event?event_name=OnKeyPressed_webservice.redled&event_source=doorpi.keyboard.from_filesystem')
        r.json()
	exit(0)
    else:
        print('Found template at position #' + str(positionNumber))
        print('The accuracy score is: ' + str(accuracyScore))
		# Transaction to DoorPi: Door Release OK. // Please modify URL below with your credentials e.g: http://door:pi@doorpi:8080
        r = requests.get('http://username:password@doorpi-url:port/control/trigger_event?event_name=OnKeyPressed_webservice.dooropen&event_source=doorpi.keyboard.from_filesystem')
	r.json()
		
    ## OPTIONAL stuff to secure the Transaction (future revisions)
    ##

    ## Loads the found template to charbuffer 1
    f.loadTemplate(positionNumber, 0x01)

    ## Downloads the characteristics of template loaded in charbuffer 1
    characterics = str(f.downloadCharacteristics(0x01))

    ## Hashes characteristics of template
    print('SHA-2 hash of template: ' + hashlib.sha256(characterics).hexdigest())

except Exception as e:
    print('Operation failed!')
    print('Exception message: ' + str(e))
    exit(1)
